# The following GCC versioning stuff shamelessly copied
# from the Fedora spec file for gcc-python-plugin

# GCC will only load plugins that were built against exactly that build of GCC
# We thus need to embed the exact GCC version as a requirement within the
# metadata.
#
# Define "gcc_vr", a variable to hold the VERSION-RELEASE string for the gcc
# we are being built against.
#
# Unfortunately, we can't simply run:
#   rpm -q --qf="%{version}-%{release}"
# to determine this, as there's no guarantee of a sane rpm database within
# the chroots created by our build system
#
# So we instead query the version from gcc's output.
#
# gcc.spec has:
#   Version: %{gcc_version}
#   Release: %{gcc_release}%{?dist}
#   ...snip...
#   echo 'Red Hat %{version}-%{gcc_release}' > gcc/DEV-PHASE

# So, given this output:
#
#   $ gcc --version
#   gcc (GCC) 4.6.1 20110908 (Red Hat 4.6.1-9)
#   Copyright (C) 2011 Free Software Foundation, Inc.
#   This is free software; see the source for copying conditions.  There is NO
#   warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# we can scrape out the "4.6.1-9" from the version line.
#
# We then have to add the dist tag; which we must hope is the same as what we
# have, unless the gcc dist tag has a "minor release bump" such as ".1"
# appended, which we have to deal with manually.
#
# The following implements the above:

%global gcc_dist %{dist}
#global gcc_dist_minor 1

%global gcc_vr %(gcc --version | python -c "import re; import sys; print(re.match(r'.*\\(Red Hat (.+)\\).*', sys.stdin.read()).group(1))")%{gcc_dist}%{?gcc_dist_minor:.%{gcc_dist_minor}}

# Define a boolean to make it easy to turn the above off, in case it fails:
%global with_hard_gcc_version_requirement 1


# Exclude ARM until upstream fixes problems that prevent successful build.
# Fedora bug:    https://bugzilla.redhat.com/show_bug.cgi?id=992138
# Upstream bug:  http://llvm.org/bugs/show_bug.cgi?id=16829
ExcludeArch: armv7hl

#global prerel rc3

%global downloadurl http://llvm.org/%{?prerel:pre-}releases/%{version}%{?prerel:/%{prerel}}

%global gcc_plugins_dir %(gcc --print-file-name=plugin)

Name:           dragonegg
Version:        3.3
Release:        %{?prerel:0.}11%{?prerel:.}%{?prerel}%{?dist}
Summary:        GCC plugin to use LLVM optimizers and code generators

Group:          Development/Languages
License:        GPLv2+
URL:            http://dragonegg.llvm.org/

# It would be nice if upstream was more consistent in naming the
# source distribution archives.  It's variously been:
#     dragonegg-source-version.tar.gz,
#     dragonegg-version-source.tar.gz
#     dragonegg-version.src.tar.gz
Source0:        %{downloadurl}/%{name}-%{version}%{?prerel}.src.tar.gz

# As supplied, the DragonEgg Makefile uses llvm-config to get linker options,
# which try to link the static LLVM libraries.  We want the shared library
# instead, so we patch the Makefile and pass in suitable LDFLAGS on the
# link command line.
Patch0:         dragonegg-3.3rc2-dyn.patch

# Correct a syntax error in include/arm/dragonegg/Target.h
# http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=715424
Patch1:         dragonegg-3.3-arm-target.patch

BuildRequires:  llvm-devel = %{version}
BuildRequires:  llvm-libs = %{version}
BuildRequires:  gcc-plugin-devel
BuildRequires:  python

%if %{with_hard_gcc_version_requirement}
Requires: gcc = %{gcc_vr}
%else
Requires: gcc
%endif

%description
DragonEgg is a GCC plugin that replaces GCC's optimizers and code generators
with those from the LLVM project.

%prep
%setup -q -n %{name}-%{version}.src
%patch0 -p1 -b .dyn 
%patch1 -p1 -b .arm-target

%build
echo "gcc_vr = %{gcc_vr}"
# FIXME needs %{optflags}
GCC=%{_bindir}/gcc make %{_smp_mflags} LDFLAGS="-L%{_libdir}/llvm -lLLVM-%{version}"

%install
mkdir -p %{buildroot}%{gcc_plugins_dir}
install -m 755 dragonegg.so %{buildroot}%{gcc_plugins_dir}


%files
%defattr(-,root,root,-)
%doc README COPYING
%{gcc_plugins_dir}/%{name}.so


%changelog
* Thu Oct 17 2013 Jakub Jelinek <jakub@redhat.com> - 3.3-11
- Rebuild for gcc-4.8.2-1.

* Sat Sep 21 2013 Eric Smith <spacewar@gmail.com> - 3.3-10
- Rebuild for gcc-4.8.1-10.fc21.

* Mon Sep 16 2013 Eric Smith <spacewar@gmail.com> - 3.3-9
- Rebuild for gcc-4.8.1-9.fc21.

* Tue Sep 10 2013 Eric Smith <spacewar@gmail.com> - 3.3-8
- Rebuild for gcc-4.8.1-8.fc21.

* Fri Sep 06 2013 Eric Smith <spacewar@gmail.com> - 3.3-7
- Rebuild for gcc-4.8.1-7.fc21.

* Thu Aug 15 2013 Eric Smith <spacewar@gmail.com> - 3.3-6
- Rebuild for gcc-4.8.1-6.fc20.

* Thu Aug 08 2013 Eric Smith <spacewar@gmail.com> - 3.3-5
- Rebuild for gcc-4.8.1-5.fc20.1.

* Sat Aug 03 2013 Eric Smith <spacewar@gmail.com> - 3.3-4
- LLVM package no longer has ExcludeArch s390 s390x ppc ppc64, so removed here
  also.
- Added patch for one ARM build problem, but still fails, so adding
  ExcludeArch arm until fixed upstream.

* Sat Aug 03 2013 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.3-3.1
- Rebuilt for https://fedoraproject.org/wiki/Fedora_20_Mass_Rebuild

* Sat Jul 20 2013 Eric Smith <spacewar@gmail.com> - 3.3-3
- Rebuild for gcc-4.8.1-5.fc20.

* Wed Jul 17 2013 Eric Smith <spacewar@gmail.com> - 3.3-2
- Rebuild for gcc-4.8.1-4.fc20.

* Tue Jun 18 2013 Eric Smith <spacewar@gmail.com> - 3.3-1
- Updated to latest upstream.

* Thu Jun 13 2013 Eric Smith <spacewar@gmail.com> - 3.3-0.4.rc3
- Rebuild for gcc-4.8.1-2.fc20.

* Fri Jun 07 2013 Eric Smith <eric@brouhaha.com> - 3.3-0.3.rc3
- Updated to latest upstream.

* Tue Jun 04 2013 Eric Smith <brouhaha@fedoraproject.org> - 3.3-0.2.rc2
- Rebuild for gcc-4.8.1-1.fc20.

* Mon Jun 03 2013 Eric Smith <eric@brouhaha.com> - 3.3-0.1.rc2
- Updated to latest upstream.

* Wed Feb 13 2013 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.1-20
- Rebuilt for https://fedoraproject.org/wiki/Fedora_19_Mass_Rebuild

* Fri Dec 07 2012 Eric Smith <eric@brouhaha.com> - 3.1-19
- Rebuild for gcc-4.7.2-9.fc19.

* Thu Nov 22 2012 Eric Smith <eric@brouhaha.com> - 3.1-18
- Fixed gcc dependency.

* Sun Nov 11 2012 Eric Smith <eric@brouhaha.com> - 3.1-17
- Rebuild for gcc-4.7.2-8.fc19.

* Fri Nov 09 2012 Eric Smith <eric@brouhaha.com> - 3.1-16
- Rebuild for gcc-4.7.2-7.fc19.

* Sat Oct 27 2012 Eric Smith <eric@brouhaha.com> - 3.1-15
- Add BuildRequires for Python. Previously it was getting pulled in as
  a dependency of something else, but that's stopped happening.

* Sat Oct 20 2012 Eric Smith <eric@brouhaha.com> - 3.1-14
- Rebuild for gcc-4.7.2-6.fc19.

* Wed Oct 10 2012 Eric Smith <eric@brouhaha.com> - 3.1-13
- Rebuild for gcc 4.7.2-4.fc19.

* Tue Oct 09 2012 Eric Smith <eric@brouhaha.com> - 3.1-12
- Rebuild for gcc 4.7.2-3.fc19.

* Sun Sep 23 2012 Eric Smith <eric@brouhaha.com> - 3.1-11
- Rebuild for gcc 4.7.2-1.fc19.

* Fri Sep 14 2012 Eric Smith <eric@brouhaha.com> - 3.1-10
- Rebuild for gcc-4.7.1-8.fc19.

* Thu Aug 16 2012 Eric Smith <eric@brouhaha.com> - 3.1-9
- Rebuild for gcc-4.7.1-7.fc19.

* Fri Aug 10 2012 Eric Smith <eric@brouhaha.com> - 3.1-8
- Rebuild for gcc-4.7.1-5.

* Thu Aug 09 2012 Eric Smith <eric@brouhaha.com> - 3.1-7
- Rebuild for libffi update.

* Sat Aug 04 2012 Eric Smith <eric@brouhaha.com> - 3.1-6
- Rebuild for gcc-4.7.1-5.fc18.

* Wed Jul 18 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.1-5
- Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild

* Mon Jul 16 2012 Eric Smith <eric@brouhaha.com> - 3.1-4
- Rebuilt for gcc-4.7.1-2.fc18.

* Mon Jul 09 2012 Eric Smith <eric@brouhaha.com> - 3.1-3
- Rebuilt for gcc-4.7.1-1.fc18.

* Tue Jun 12 2012 Eric Smith <eric@brouhaha.com> - 3.1-2
- Rawhide gcc still has fc17 dist tag, so temporary changed the
  gcc dependency macros to use fc17.

* Wed May 23 2012 Eric Smith <eric@brouhaha.com> - 3.1-1
- Updated to latest upstream.
- Added ExcludeArch to avoid building on architectures for which LLVM is
  not built.  It seems to me that it should be possible to build LLVM on
  the excluded architectures for use as a cross-compiler host only, but this
  isn't being done.  Note that we currently don't build for any EPEL
  because no EPEL provides a new enough LLVM.  Maybe someday.

* Sat Dec 03 2011 Eric Smith <eric@brouhaha.com> - 3.0-1
- Update to upstream final release

* Fri Dec 02 2011 Eric Smith <eric@brouhaha.com> - 3.0-0.1.rc3
- Initial version
